﻿using System;
using System.Globalization;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;

namespace LommeReimar
{
    public partial class LommeReimarViewModel : ObservableObject
    {
        public enum Operator
        {
            PLUS = '+',
            MINUS = '-',
            TIMES = '×',
            DIVIDE = '÷',
        }

        private decimal? firstNumber;

        [ObservableProperty]
        private decimal currentNumber;

        [ObservableProperty]
        private Operator? currentOperator;

        private bool isDecimal;

        public LommeReimarViewModel()
        {
            firstNumber = null;
            currentNumber = 0;
            isDecimal = false;
            currentOperator = null;
        }

        [RelayCommand]
        public void NumberClick(string number)
        {
            if (isDecimal)
            {
                string str = CurrentNumber.ToString();
                if (!str.Contains(NumberFormatInfo.CurrentInfo.NumberDecimalSeparator))
                    str += NumberFormatInfo.CurrentInfo.NumberDecimalSeparator;

                CurrentNumber = decimal.Parse(str + number);
            }
            else
            {
                CurrentNumber *= 10;
                CurrentNumber += decimal.Parse(number);
            }
        }

        [RelayCommand]
        public void OperatorClick(string op)
        {
            switch (op)
            {
                case "+":
                    CurrentOperator = Operator.PLUS;
                    break;
                case "-":
                    CurrentOperator = Operator.MINUS;
                    break;
                case "×":
                    CurrentOperator = Operator.TIMES;
                    break;
                case "÷":
                    CurrentOperator = Operator.DIVIDE;
                    break;
                default:
                    return;
            }

            firstNumber = CurrentNumber;
            CurrentNumber = 0;
            isDecimal = false;
        }

        [RelayCommand]
        public void EqualsClick()
        {
            if (firstNumber is null) return;

            switch (CurrentOperator)
            {
                case Operator.PLUS:
                    CurrentNumber = (decimal)(firstNumber + CurrentNumber);
                    break;
                case Operator.MINUS:
                    CurrentNumber = (decimal)(firstNumber - CurrentNumber);
                    break;
                case Operator.TIMES:
                    CurrentNumber = (decimal)(firstNumber * CurrentNumber);
                    break;
                case Operator.DIVIDE:
                    CurrentNumber = (decimal)(firstNumber / CurrentNumber);
                    break;
            }
        }

        [RelayCommand]
        public void InvertClick()
        {
            CurrentNumber = -CurrentNumber;
        }

        [RelayCommand]
        public void ClearClick()
        {
            firstNumber = null;
            CurrentNumber = 0;
            CurrentOperator = null;
            isDecimal = false;
        }

        [RelayCommand]
        public void DecimalClick()
        {
            isDecimal = true;
        }

        [RelayCommand]
        public void PercentageClick()
        {
            if (firstNumber is null || CurrentOperator is null) return;

            switch (CurrentOperator)
            {
                case Operator.PLUS:
                    CurrentNumber = (decimal)(firstNumber + CurrentNumber / 100 * firstNumber);
                    break;
                case Operator.MINUS:
                    CurrentNumber = (decimal)(firstNumber - CurrentNumber / 100 * firstNumber);
                    break;
                case Operator.TIMES:
                    CurrentNumber = (decimal)(firstNumber * CurrentNumber / 100);
                    break;
                case Operator.DIVIDE:
                    CurrentNumber = (decimal)(firstNumber / CurrentNumber * 100);
                    break;
            }
        }
    }
}
